/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.config;

import java.io.File;
import java.net.URL;
import java.util.Properties;

/** Entry for some config file
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: ConfigEntry.java,v 1.1 2004/12/08 22:57:52 masterludo Exp $
 * @created 12 novembre 2003, 16:17
 */
public class ConfigEntry {
  
  /** Holds the path to the config file */
  private String path;
  
  /** Holds the name of the config file */
  private String filename;
  
  /** Holds the properties */
  private Properties properties;
  
  /** Holds the key for the log file name */
  public static final String LOG_FILE_KEY = "log.file";
  
  /**
   * Creates a new instance of ConfigEntry
   * @param aPath  the path to the config file
   * @param aFilename  the name of the config file
   */
  public ConfigEntry(String aPath, String aFilename) {
    setConfigPath(aPath);
    setFilename(aFilename);
    loadConfig();
  }
  
  /** Load the config file
   */
  protected void loadConfig() {
    Properties props = new Properties();
    try {
      URL configUrl = new File(getConfigPath() + getFilename()).toURL();
      props.load( configUrl.openStream() );
      setProperties(props); // do first, important otherwise other classes can't use properties
    } catch (Exception ex) {
      setProperties(null);
      // TODO -> log this
      ex.printStackTrace();
    }
  }
  
  /** Gets the path to the config file
   * @return  the path to the config file
   */
  public String getConfigPath() {
    return path;
  }
  
  /** Sets the path to the config file
   * @param aPath  the path to the config file, must not be null
   */
  protected void setConfigPath(String aPath) {
    if (null == aPath) {
      throw new IllegalArgumentException("The config file path must not be null");
    }
    String separator = System.getProperty("file.separator");
    if ( !aPath.endsWith(separator) ) {
      aPath = aPath + separator;
    }
    path = aPath;
  }
  
  /** Gets the name of the config file
   * @return  the name of the config file
   */
  public String getFilename() {
    return filename;
  }
  
  /** Sets the name of the config file
   * @param aName  the name of the config file, must not be null
   */
  protected void setFilename(String aName) {
    if (null == aName) {
      throw new IllegalArgumentException("The name of the config file must not be null");
    }
    filename = aName;
  }
  
  /** Gets the properties for this config
   * @return  some properties, can be null if the config could not be loaded
   */
  public Properties getProperties() {
    return properties;
  }
  
  /** Sets the properties for this config
   * @param someProperties  some properties, can be null if the config could not be loaded
   */
  protected void setProperties(Properties someProperties) {
    properties = someProperties;
  }
  
  /** Get the property with the given key
   * @param aKey  the key to retrieve the property
   * @return  the property corresponding to the given key or null if there is not such a key
   */
  public String getProperty(String aKey) {
    String result = null;
    if (null != getProperties()) {
      result = getProperties().getProperty(aKey);
    }
    return result;
  }
  
}
