/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.logging;

import java.io.File;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Handler;
import java.util.logging.Logger;
import java.util.logging.StreamHandler;
import org.ludo.config.ConfigEntry;

/** Default implementation of the SafePeerLogger.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: DefaultMiniLogger.java,v 1.1 2004/12/08 22:57:52 masterludo Exp $
 * @created 11. November 2003, 08:24
 */
public class DefaultMiniLogger implements MiniLogger {
  
  /** Holds the map of all loggers active */
  private static Map allLoggers = new HashMap();
  
  /** Holds the logger */
  private Logger logger;
  
  /** Holds the name of the logger */
  private String name;
  
  /** Holds the flag for the logging enable state */
  private static boolean loggingEnabled = true;
  
  /**
   * Creates a new instance of DefaultMiniLogger.
   * @param aName  the name of this mini logger.
   */
  protected DefaultMiniLogger(String aName) {
    setName(aName);
    setLogger(Logger.getLogger(getName()));
    getLogger().setUseParentHandlers(false);
  }
  
  /**
   * Checks whether the logging is enabled or not.
   * @return  true if the logging is enabled.
   */
  public static boolean isLoggingEnabled() {
  	return loggingEnabled;
  }
  
  /**
   * Sets whether the logging is enabled or not.
   * @param aFlag  true if the logging is enabled.
   */
  public static void setLoggingEnabled(boolean aFlag) {
  	loggingEnabled = aFlag;
  }
  
  /**
   * Creates a logger instance.
   * @param anIdentifier  the identifier for the logger to create.
   * @param anEntry  a ConfigEntry for the logger to create, must not be null.
   */
  public static void createLoggerInstance(String anIdentifier, ConfigEntry anEntry) {
    MiniLogger log = new DefaultMiniLogger(anIdentifier);
    if (null == allLoggers.get(anIdentifier)) {
	    try {
	      String logfile = anEntry.getProperty(ConfigEntry.LOG_FILE_KEY);
	      if (null == logfile) {
	        logfile = "minilogger.log";
	      }
	      File configFile = new File(anEntry.getConfigPath() + logfile);
	      FileOutputStream fos = new FileOutputStream(configFile, true);
	      StreamHandler handler = new StreamHandler(fos, new MiniFormatter());
	      log.addHandler(handler);
	    } catch (Exception ex) {
	      // TODO -> HOW TO LOG THIS???
	      ex.printStackTrace();
	    }
	    allLoggers.put(anIdentifier, log);
    }
  }
  
  /** Get a logger instance for the class with the given name
   * @param anIdentifier  the identifier of the logger
   * @return  a MiniLogger, cannot be null
   */
  public static MiniLogger getInstance(String anIdentifier) {
    return (MiniLogger)allLoggers.get(anIdentifier);
    
  }
  
  /** Get a logger instance for the class with the given name
   * @param aClassName  the name of the class to return a logger for
   * @return  a SafePeerLogger, cannot be null
   */
  /*
  public static MiniLogger getInstance(String aClassName, File aLogFile) {
    MiniLogger log = (MiniLogger)allLoggers.get(aClassName);
    if (null == log) {
      log = new DefaultMiniLogger(aClassName);
      try {
        FileOutputStream fos = new FileOutputStream(aLogFile, true);
        StreamHandler handler = new StreamHandler(fos, new MiniFormatter());
        log.addHandler(handler);
      } catch (Exception ex) {
        // HOW TO LOG THIS???
        ex.printStackTrace();
      }
      allLoggers.put(aClassName, log);
    }
    return log;
  }
  */
  
  /** Gets the name of the logger
   * @return  the name of the logger
   */
  public String getName() {
    return name;
  }
  
  /** Sets the name of the logger
   * @param aName  the name of the logger
   */
  protected void setName(String aName) {
    name = aName;
  }
  
  /** Logs the given message as info
   * @param aMessage  the message to log
   */
  public void info(String aMessage) {
    if (isLoggingEnabled()) {
      getLogger().info(aMessage);
      flushLogger();
    }
  }
  
  /** Logs the given message as info
   * @param aMessage  the message to log
   */
  public void warn(String aMessage) {
    if (isLoggingEnabled()) {
      getLogger().warning(aMessage);
      flushLogger();
    }
  }
    
  /** Logs the given message as info
   * @param aMessage  the message to log
   */
  public void error(String aMessage) {
    if (isLoggingEnabled()) {
      getLogger().severe(aMessage);
      flushLogger();
    }
  }
  
  /**
   * Flushes the logger by flushing all its handlers.
   */
  protected void flushLogger() {
    Handler[] handlers = getLogger().getHandlers();
    if (null != handlers) {
      int len = handlers.length;
      for (int i = 0; i < len; i++) {
        handlers[i].flush();
      }
    }
  }
  
  /** Gets the logger
   * @return  a Logger, cannot be null
   */
  protected Logger getLogger() {
    return logger;
  }
  
  /** Sets the logger
   * @param aLogger  a Logger, must not be null
   */
  protected void setLogger(Logger aLogger) {
    if (null == aLogger) {
      throw new IllegalArgumentException("The logger may not be null");
    }
    logger = aLogger;
  }
  
  /** Add the given handler to the logger
   * @param aHandler  a handler to add, must not be null
   */
  public void addHandler(Handler aHandler) {
    getLogger().addHandler(aHandler);
  }
  
}
