package org.ludo.net;

import java.net.UnknownHostException;
import org.ludo.util.StringUtils;

/**
 * Memory optimized IpRange. Taken from Azureus' code with the approval of parg.
 * @author <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert </a>
 * @revision $Id: IpRange.java,v 1.1 2004/12/08 22:57:52 masterludo Exp $
 * @created Oct 27, 2004
 */
public class IpRange {
	private byte[] description;

	private Object startIp; // Integer if value, String/null otherwise

	private Object endIp; // Integer if value, String/null otherwise

	/**
	 * 
	 * @param _description
	 * @param _startIp
	 * @param _endIp
	 */
	public IpRange(String _description, String _startIp, String _endIp) {
		if (_startIp == null || _endIp == null) {
			throw (new RuntimeException(
					"Invalid start/end values - null not supported"));
		}
		startIp = _startIp.trim();
		endIp = _endIp.trim();
		internDescription(_description);
		checkValid();
	}

	public void checkValid() {
		if (startIp != null && endIp != null) {
			if (startIp instanceof String) {
				try {
					startIp = new Integer(StringUtils.addressToInt((String) startIp));
				} catch (UnknownHostException e) {
				}
			}
			if (endIp instanceof String) {
				try {
					endIp = new Integer(StringUtils.addressToInt((String) endIp));
				} catch (UnknownHostException e) {
				}
			}
		}
	}

	/**
	 * 
	 * @return
	 */
	public boolean isValid() {
		if (startIp instanceof Integer && endIp instanceof Integer) {
			long start_address = ((Integer) startIp).intValue();
			long end_address = ((Integer) endIp).intValue();
			if (start_address < 0) {
				start_address += 0x100000000L;
			}
			if (end_address < 0) {
				end_address += 0x100000000L;
			}
			return (end_address >= start_address);
		}
		return (false);
	}

	/**
	 * 
	 * @param ipAddress
	 * @return
	 */
	public boolean isInRange(String ipAddress) {
		if (!isValid()) {
			return false;
		}
		try {
			long int_address = StringUtils.addressToInt(ipAddress);
			if (int_address < 0) {
				int_address += 0x100000000L;
			}
			long start_address = ((Integer) startIp).intValue();
			long end_address = ((Integer) endIp).intValue();
			if (start_address < 0) {
				start_address += 0x100000000L;
			}
			if (end_address < 0) {
				end_address += 0x100000000L;
			}
			return (int_address >= start_address && int_address <= end_address);
		} catch (UnknownHostException e) {
			return (false);
		}
	}

	/**
	 * 
	 * @return
	 */
	public String getDescription() {
		return (externDescription());
	}

	/**
	 * 
	 * @param str
	 */
	public void setDescription(String str) {
		internDescription(str);
	}

	// private static Map word_map = new HashMap();
	private final static byte[][] frequent_words = {"ap2p".getBytes(),
			"city".getBytes(), "of".getBytes(), "proxy".getBytes(),
			"spider".getBytes(), ".com".getBytes(), "ads".getBytes(),};

	/**
	 * 
	 * @param description_str
	 */
	protected void internDescription(String description_str) {
		if (description_str == null) {
			description = null;
			return;
		}
		byte[] bytes = description_str.toLowerCase().getBytes();
		int word_start = 0;
		description = new byte[bytes.length];
		int desc_pos = 0;
		for (int i = 0; i < bytes.length + 1; i++) {
			byte b = i == bytes.length ? (byte) ' ' : bytes[i];
			// kill any existing characters in the space we're using for frequent
			// words
			if (b < 0) {
				b = (byte) '_';
			}
			if (b == ' ' || b == '.' || b == '-') {
				int word_len = i - word_start;
				boolean hit = false;
				if (word_len > 1) {
					for (int j = 0; j < frequent_words.length; j++) {
						byte[] fw = frequent_words[j];
						if (fw.length == word_len) {
							hit = true;
							for (int k = word_start; k < i; k++) {
								if (bytes[k] != fw[k - word_start]) {
									hit = false;
									break;
								}
							}
						}
						if (hit) {
							description[desc_pos++] = (byte) (128 + j);
							break;
						}
					}
				}
				if (!hit) {
					for (int j = word_start; j < i; j++) {
						description[desc_pos++] = bytes[j] < 0 ? (byte) '_' : bytes[j];
					}
				}
				if (i < bytes.length) {
					description[desc_pos++] = b;
				}
				word_start = i + 1;
			}
		}
		if (desc_pos < description.length) {
			byte[] d = new byte[desc_pos];
			System.arraycopy(description, 0, d, 0, desc_pos);
			description = d;
		}
	}

	protected String externDescription() {
		if (description == null) {
			return (null);
		}
		StringBuffer res = new StringBuffer();
		for (int i = 0; i < description.length; i++) {
			byte b = description[i];
			if (b < 0) {
				res.append(new String(frequent_words[128 + b]));
			} else {
				res.append((char) b);
			}
		}
		return (res.toString());
	}

	/**
	 * 
	 * @return
	 */
	public String getStartIp() {
		return (startIp instanceof Integer ? StringUtils
				.intToAddress(((Integer) startIp).intValue()) : (String) startIp);
	}

	/**
	 * 
	 * @param str
	 */
	public void setStartIp(String str) {
		if (str == null) {
			throw (new RuntimeException("Invalid start value - null not supported"));
		}
		if (str.equals(getStartIp())) {
			return;
		}
		startIp = str;
		checkValid();
	}

	/**
	 * 
	 * @return
	 */
	public String getEndIp() {
		return (endIp instanceof Integer ? StringUtils.intToAddress(((Integer) endIp)
				.intValue()) : (String) endIp);
	}

	/**
	 * 
	 * @param str
	 */
	public void setEndIp(String str) {
		if (str == null) {
			throw (new RuntimeException("Invalid end value - null not supported"));
		}
		if (str.equals(getEndIp())) {
			return;
		}
		endIp = str;
		checkValid();
	}
	

	/**
	 * 
	 */
	public String toString() {
		return (getDescription() + " : " + getStartIp() + " - " + getEndIp());
	}
}