/*
 * SafePeer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;
import org.ludo.blocklist.BlocklistConstants;
import org.ludo.blocklist.BlocklistManager;
import org.ludo.config.ConfigConstants;
import org.ludo.config.ConfigEntry;
import org.ludo.safepeer.cache.DefaultEvilIPCache;
import org.ludo.safepeer.cache.EvilIPCache;
import org.ludo.safepeer.database.DatabaseDataRetriever;
import org.ludo.safepeer.database.PeerGuardianDatabaseDataRetriever;
import org.ludo.safepeer.parser.DatabaseDataParser;
import org.ludo.safepeer.parser.PeerGuardianDatabaseDataParser;
import org.ludo.util.URLParser;

/** SafePeer is the main class providing access to the database retriever, parser and ip cache
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: SafePeer.java,v 1.5 2004/12/12 10:06:41 masterludo Exp $
 * @created 10. November 2003, 09:11
 */
public class SafePeer {
  
  /** Holds the DatabaseDataRetriever */
  private DatabaseDataRetriever databaseDataRetriever;
  
  /** Holds the DatabaseDataParser */
  private DatabaseDataParser databaseDataParser;
  
  /** Holds the EvilIPCache */
  private EvilIPCache cache;
  
  /** Holds the config entry */
  private ConfigEntry configEntry;
  
  /** Holds the log identifier */
  public static final String LOG_IDENTIFIER = "SafePeerLogger";
  
  /** The version number */
  public static final String INTERNAL_REVISION = "2.5.1";
  
  /** Creates a new SafePeer with a PeerGuardianDatabaseDataRetriever, a PeerGuardianDatabaseDataParser and a DefaultEvilIPCache.
   * @param aConfig  the ConfigEntry to use for the configuration.
   */
  public SafePeer(ConfigEntry aConfig) {
    setConfigEntry(aConfig);
    parseConfig();
    setDatabaseDataRetriever( new PeerGuardianDatabaseDataRetriever() );
    setDatabaseDataParser(new PeerGuardianDatabaseDataParser());
    setCache(DefaultEvilIPCache.getInstance());
    initializeSafePeer();
  }
  
  /** Creates a new SafePeer with given DatabaseDataRetriever and DatabaseDataParser.
	 * @param aConfig  the ConfigEntry to use for the configuration.
   * @param aRetriever  a DatabaseDataRetriever, must not be null. Default is PeerGuardianDatabaseDataRetriever.
   * @param aParser  a DatabaseDataParser, must not be null. Default is PeerGuardianDatabaseDataParser.
   * @param aCache  an EvilIPCache containing the evil IPs.
   */
  /*
  public SafePeer(ConfigEntry aConfig, DatabaseDataRetriever aRetriever, DatabaseDataParser aParser, EvilIPCache aCache) {
    setConfigEntry(aConfig);
    parseConfig();
    setDatabaseDataRetriever(aRetriever);
    setDatabaseDataParser(aParser);
    setCache(aCache);
    initializeSafePeer();
  }
  */
  
	/**
	 * Parses the config file to configure the logger.
	 */
  protected void parseConfig() {
    //DefaultMiniLogger.createLoggerInstance(LOG_IDENTIFIER, getConfigEntry());
    
    boolean enableLogging = false;
    try {
      enableLogging = Boolean.valueOf(getConfigEntry().getProperty(SafepeerConfigConstants.LOGGING_KEY)).booleanValue();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    //DefaultMiniLogger.setLoggingEnabled(enableLogging);
    //DefaultMiniLogger.getInstance(LOG_IDENTIFIER).info("Config file successfuly loaded");
  }
  
  /** Initialize the SafePeer
   */
  protected void initializeSafePeer() {
  	boolean isSafePeerEnabled = Boolean.valueOf( getConfigEntry().getProperty(SafepeerConfigConstants.ENABLE_SAFEPEER) ).booleanValue();
  	//MiniLogger log = DefaultMiniLogger.getInstance(LOG_IDENTIFIER);
  	if (isSafePeerEnabled) {
  		boolean isBlocklistManagerEnabled = Boolean.valueOf( getConfigEntry().getProperty(SafepeerConfigConstants.ENABLE_BLOCKLIST_MANAGER) ).booleanValue();
  		List result = null;
  		if (isBlocklistManagerEnabled) {
  			ConfigEntry blocklistEntry = new ConfigEntry(getConfigEntry().getConfigPath(), BlocklistConstants.CONFIG_FILE_NAME);
  			// TODO
  			BlocklistManager.getInstance().loadBlocklist(blocklistEntry);
  		} else {
  			result = parseDatabases();
  		}
  		if (null != result && !result.isEmpty()) {
  			DefaultEvilIPCache.getInstance().addEvilPeerEntry(result);
  		}
  	} else {
  		//log.info("SafePeer is currently disabled. Change the entry in the config file and restart to enable it.");
  	}
  }
  
  
  protected List parseDatabases() {
  	List result = Collections.EMPTY_LIST;
  	//MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
  	
  	String cachename = getConfigEntry().getConfigPath()
			+ getConfigEntry().getProperty(SafepeerConfigConstants.CACHE_FILE_KEY); 
  	File cacheFile = new File(cachename);
  	File tmpCache = new File(cachename + ".old");
  	if (cacheFile.exists()) {
  		try {
  			FileOutputStream fos = new FileOutputStream(tmpCache);
  			FileInputStream fis = new FileInputStream(cacheFile);
	  		URLParser.readInputStream(fos, fis);
	  		fis.close();
	  		fos.close();
  		} catch (IOException ioe) {
  			ioe.printStackTrace();
  		}
  	}
  	result = parsePrimaryDatabase();
  	if (null == result || result.isEmpty()) {
  		result = parseSecondaryDatabases();
  		if (null == result || result.isEmpty()) {
  			if (null != tmpCache) {
  				try {
	  				FileInputStream fis = new FileInputStream(tmpCache);
	  				FileOutputStream fos = new FileOutputStream(cacheFile);
	  				URLParser.readInputStream(fos, fis);
  				} catch (Exception ex) {
  					ex.printStackTrace();
  				}
  			}
  			result = parseCache();
  		}
  	}
  	if (null != tmpCache && tmpCache.exists()) {
  		tmpCache.delete();
  	}
  	return result;
  }
  
  protected List parsePrimaryDatabase() {
  	List result = Collections.EMPTY_LIST;
  	//MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
  	try {
  		// TODO -> check for valid entry
  		long connectTimeout = Long.valueOf(getConfigEntry().getProperty(ConfigConstants.SOCKET_CONNECT_TIMEOUT)).longValue() * 1000;
  		// TODO -> check for valid entry
  		long readTimeout = Long.valueOf(getConfigEntry().getProperty(ConfigConstants.SOCKET_READ_TIMEOUT)).longValue() * 1000;
  		
  		String file = getConfigEntry().getConfigPath()
				+ getConfigEntry().getProperty(SafepeerConfigConstants.CACHE_FILE_KEY);
  		File cacheFile = new File(file);
  		FileOutputStream fos = null;
  		try {
  			fos = new FileOutputStream(cacheFile);
  		} catch (FileNotFoundException fnfe) {
  			fnfe.printStackTrace();
  		}
  		String primaryDB = getConfigEntry().getProperty(SafepeerConfigConstants.MAIN_DB_PROPERTY_KEY);
  		List files = getDatabaseDataRetriever().retrieveDataFiles(new URL(primaryDB), getConfigEntry());
  		if (null != files) {
		    result = getDatabaseDataParser().parse(files, fos);
		    //DefaultEvilIPCache.getInstance().addEvilPeerEntry(result);
		    //log.info(DefaultEvilIPCache.getInstance().getEvilPeerEntries().size() + " entries are in the DB");
  		}
  		fos.close();
		} catch (Exception ex) {
			// TODO
		}
		return result;
  }
  
  protected List parseSecondaryDatabases() {
  	List result = new ArrayList(5000);
  	//MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
  	try {
  		String secondaryDBs = getConfigEntry().getProperty(SafepeerConfigConstants.SECONDARY_DB_PROPERTY_KEY);
  		String file = getConfigEntry().getConfigPath()
				+ getConfigEntry().getProperty(SafepeerConfigConstants.CACHE_FILE_KEY);
			File cacheFile = new File(file);
			FileOutputStream fos = null;
			try {
				fos = new FileOutputStream(cacheFile);
			} catch (FileNotFoundException fnfe) {
				fnfe.printStackTrace();
			}
  		StringTokenizer tokenizer = new StringTokenizer(secondaryDBs, ",");
  		while (tokenizer.hasMoreTokens()) {
	  		List files = getDatabaseDataRetriever().retrieveDataFiles(new URL(tokenizer.nextToken()), getConfigEntry());
	  		if (null != files) {
			    result.addAll(getDatabaseDataParser().parse(files, fos));
			    //DefaultEvilIPCache.getInstance().addEvilPeerEntry(result);
			    //log.info(DefaultEvilIPCache.getInstance().getEvilPeerEntries().size() + " entries are in the DB");
			    //break; // one result was found, no need to go on...
	  		}
  		}
  		fos.close();
		} catch (Exception ex) {
			// TODO
		}
		return result;
  }
  
  /** Retrieve the data from the cache file
   * @return  a list of parsed files.
   */
  protected List parseCache() {
  	List result = Collections.EMPTY_LIST;
    //MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
    try {
  		String file = getConfigEntry().getConfigPath() + getConfigEntry().getProperty(SafepeerConfigConstants.CACHE_FILE_KEY);
  		File f = new File(file);
  		if (f.exists()) {
	      List files = getDatabaseDataRetriever().retrieveDataFiles(f.toURL(), getConfigEntry());
	      if (null != files) {
			    result = getDatabaseDataParser().parse(files, null);
		      //DefaultEvilIPCache.getInstance().addEvilPeerEntry(result);
		      //log.info("Data successfuly retrieved from cache");
	      }
  		} else {
  			//log.warn("No data could be retrieved from cache, no cache file exists");
  		}
    } catch (Exception ex) {
      //log.warn("No data could be retrieved from cache");
      //log.warn(ex.toString());
    }
    return result;
  }
  
  /** Check if the given peer is safe, that is it is not in the cache
   * @param anIp  the IP address as string of the peer to check
   * @return  true if the peer is safe, otherwise false
   */
  public boolean isSafePeer(String anIp) {
    return !getCache().isEvil(anIp);
  }
  
  /**
   * Updates the EvilIPCache by retrieving once again from the DatabaseDataRetriever.
   * @return  true of the update was successful, otherwise false (old entries will remain).
   */
  public synchronized boolean updateCache() {
  	boolean result = false;
  	
  	boolean isSafePeerEnabled = Boolean.valueOf( getConfigEntry().getProperty(SafepeerConfigConstants.ENABLE_SAFEPEER) ).booleanValue();
  	if (isSafePeerEnabled) {
    	getCache().clear();
    	boolean isBlocklistManagerEnabled = Boolean.valueOf( getConfigEntry().getProperty(SafepeerConfigConstants.ENABLE_BLOCKLIST_MANAGER) ).booleanValue();
  		if (isBlocklistManagerEnabled) {
  			ConfigEntry blocklistEntry = new ConfigEntry(getConfigEntry().getConfigPath(), BlocklistConstants.CONFIG_FILE_NAME);
  			result = BlocklistManager.getInstance().loadBlocklist(blocklistEntry);
  		} else {
  			List newEntries = parseDatabases();
  			if (null != newEntries && !newEntries.isEmpty()) {
  				getCache().addEvilPeerEntry(newEntries);
      		result = true;
      	}
  		}
  	}
  	return result;
  }
  
  /** Get the evil IP cache
   * @return  the evil IP cache, cannot be null
   */
  public EvilIPCache getCache() {
    return cache;
  }
  
  /** Set the evil IP cache
   * @param aCache  the evil IP cache, must not be null
   */
  protected void setCache(EvilIPCache aCache) {
    cache = aCache;
  }
  
  /** Gets the DatabaseDataRetriever to use to retrieve the evil IPs
   * @return  a DatabaseDataRetriever, cannot be null. Default is PeerGuardianDatabaseDataRetriever
   */
  protected DatabaseDataRetriever getDatabaseDataRetriever() {
    return databaseDataRetriever;
  }
  
  /** Sets the DatabaseDataRetriever to use to retrieve the evil IPs
   * @param aRetriever  a DatabaseDataRetriever, must not be null. Default is PeerGuardianDatabaseDataRetriever
   */
  protected void setDatabaseDataRetriever(DatabaseDataRetriever aRetriever) {
    if (null == aRetriever) {
      throw new IllegalArgumentException("The DatabaseDataRetriever may not be null");
    }
    databaseDataRetriever = aRetriever;
  }
  
  /** Gets the DatabaseDataParser to parse to retrieved evil IPs
   * @return  a DatabaseDataParser, cannot be null. Default is PeerGuardianDatabaseDataParser
   */
  protected DatabaseDataParser getDatabaseDataParser() {
    return databaseDataParser;
  }
  
  /** Sets the DatabaseDataParser to parse to retrieved evil IPs
   * @param aParser  a DatabaseDataParser, must not be null. Default is PeerGuardianDatabaseDataParser
   */
  protected void setDatabaseDataParser(DatabaseDataParser aParser) {
    if (null == aParser) {
      throw new IllegalArgumentException("The DatabaseDataParser may not be null");
    }
    databaseDataParser = aParser;
  }
  
  /** Gets the ConfigEntry
   * @return  a ConfigEntry, cannot be null
   */
  public ConfigEntry getConfigEntry() {
    return configEntry;
  }
  
  /** Sets the ConfigEntry
   * @param anEntry  the ConfigEntry to set, must not be null
   */
  protected void setConfigEntry(ConfigEntry anEntry) {
    configEntry = anEntry;
  }
  
  /*
  public static void main(String[] args) {
  	ConfigEntry entry = new ConfigEntry("c:\\CVS_Projects\\ludo_sources\\", SafepeerConfigConstants.CONFIG_FILE_NAME);
  	SafePeer safepeer = new SafePeer(entry);
  	System.out.println(safepeer.getCache().getEvilPeerEntries().size() + " entries were found");
  }
  */
  
}
