/*
 * SafePeer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer.cache;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.ludo.net.IpRange;

/** Default implementation of the evil IPs cache.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: DefaultEvilIPCache.java,v 1.1 2004/12/08 22:57:50 masterludo Exp $
 * @created 10. November 2003, 22:39
 */
public class DefaultEvilIPCache implements EvilIPCache {
  
  /** Holds the singleton instance of this class */
  private static DefaultEvilIPCache instance;
  
  /** Holds the cache */
  private List cache;

  /** Holds the cache of already blocked IPs */
  private List blockedCache;
  
  /** Creates a new EvilIPCache
   */
  protected DefaultEvilIPCache() {
    setCache(new ArrayList(1000));
    setBlockedCache(new ArrayList(100));
  }

  /** Returns the singleton instance of this class
   * @return  the singleton instance of this class, cannot be null
   */
  public static DefaultEvilIPCache getInstance() {
    if (null == instance) {
      instance = new DefaultEvilIPCache();
    }
    return instance;
  }
  
  /** Add an EvilPeerEntry with given organization and lower and upper IP bounds
   * @param anOrganization  the organization or description corresponding to the IP range
   * @param aLowerIPBound the lower IP bound as string of the range
   * @param aGreaterIPBound the upper IP bound as string of the range
   */
  public void addEvilPeerEntry(String anOrganization, String aLowerIPBound, String aGreaterIPBound) {
    try {
      IpRange entry =
      	new IpRange(anOrganization, aLowerIPBound, aGreaterIPBound);
      addEvilPeerEntry(entry);
    } catch (Exception ex) {
    	/*
      MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
      log.warn("Could not add EvilPeerEntry lowerBound: '" + aLowerIPBound + "', upperBound: '" + aGreaterIPBound + "'");
      log.warn(ex.toString());
      */
    }
  }
  
  /** Add the given EvilPeerEntry to the cache
   * @param anEntry  the EvilPeerEntry to add to the cache
   */
  public void addEvilPeerEntry(IpRange anEntry) {
    getCache().add(anEntry);
  }
  
  public void addRange(IpRange aRange) {
  	getCache().add(aRange);
  }
  
  /** Add the given list EvilPeerEntries to the cache
   * @param someEntries  the EvilPeerEntries to add to the cache, should not be null
   */
  public void addEvilPeerEntry(List someEntries) {
    if (null != someEntries) {
    	/**
      Iterator it = someEntries.iterator();
      while (it.hasNext()) {
        addEvilPeerEntry( (EvilPeerEntry)it.next() );
      }
      */
    	getCache().addAll(someEntries);
    }
  }
  
  /** Check if the given IP if is an evil IP known in the DB
   * @param anIp  the IP to check
   * @return  true if the IP is evil, otherwise false
   */
  public boolean isEvil(String anIp) {
    boolean result = false;
    if ( getBlockedCache().contains(anIp) ) {
      result = true;
    } else {
      try {
        InetAddress addr = InetAddress.getByName(anIp);
        Iterator it = getCache().iterator();
        while (it.hasNext()) {
        	IpRange entry = (IpRange)it.next();
          result = entry.isInRange(addr.getHostAddress());
          if (result) {
            break;
          }
        }
      } catch (Exception ex) {
      	/*
        MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
        log.warn("An exception occured while trying to check if the IP is evil");
        log.warn(ex.toString());
        */
      }
    }
    
    if (result) {
      getBlockedCache().add(anIp);
    }
    return result;
  }
  
  
  /** Gets the current cache.
   * @return  a list of EvilPeerEntries (in the cache), cannot be null.
   */
  protected List getCache() {
    return cache;
  }  
  
  /** Sets the current cache.
   * @param aCache  a list of EvilPeerEntries (in the cache), must not be null.
   */
  protected void setCache(List aCache) {
    if (null == aCache) {
      throw new IllegalArgumentException("The cache list may not be null");
    }
    cache = aCache;
  }
  
  /** Gets the list of all EvilPeerEntries in the cache.
   * @return  an unmodifiable list of all EvilPeerEntries in the cache.
   */
  public List getEvilPeerEntries() {
    return Collections.unmodifiableList(getCache());
  }

  /** Gets the cache of already blocked IPs
   * @return  a list of IPs as string, cannot be null
   */
  protected List getBlockedCache() {
    return blockedCache;
  }
  
  /** Sets the cache of already blocked IPs
   * @param aCache  a list of IPs as string, must not be null
   */
  protected void setBlockedCache(List aCache) {
    if (null == aCache) {
      throw new IllegalArgumentException("The cache list may not be null");
    }
    blockedCache = aCache;
  }
  
  /** Gets the cache of IPs that have already been blocked
   * @return  an unmodifiable list of IPs as string, cannot be null
   */
  public List retrieveBlockedCache() {
    return Collections.unmodifiableList(getBlockedCache());
  }
  
  /** Clears the cache.
   */
  public void clear() {
  	getCache().clear();
  	getBlockedCache().clear();
  }
}
