/*
 * SafePeer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ludo.safepeer.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import org.ludo.net.IpRange;

/** Implementation of the DatabaseDataParser interface for the PeerGuardian database.
 * @author  <a href="mailto:masterludo@gmx.net">Ludovic Kim-Xuan Galibert</a>
 * @revision $Id: PeerGuardianDatabaseDataParser.java,v 1.1 2004/12/08 22:57:50 masterludo Exp $
 * @created 10. November 2003, 15:16
 */
public class PeerGuardianDatabaseDataParser implements DatabaseDataParser {
	
	private static final byte[] LINE_FEED = "\n".getBytes();
  
  /** Parse the data retrieved by the given DatabaseDataRetriever
   * @param someData  the DatabaseData, should be some PeerGuardianData
   * @return  a list of EvilPeerEntry objects, cannot be null.
   * @throws IllegalArgumentException  if the DatabaseData is not some PeerGuardianData.
   */
  public List parse(List someFiles, OutputStream aCache) throws IllegalArgumentException {
  	/*
  	System.out.println("memory before parse: "
				+ ((double)Runtime.getRuntime().totalMemory() / 1000000D) + "M");
		*/
    List result = null;
    //MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
    try {
      //ArrayList peerGuardianEntries = new ArrayList(1000);
      result = new ArrayList(1000);
      if (null != someFiles && !someFiles.isEmpty()) {
      	for (Iterator it = someFiles.iterator(); it.hasNext();) {
      		String file = it.next().toString();
      		File toRead = new File(file);
	      	FileReader fReader = new FileReader(toRead);
	      	BufferedReader bReader = new BufferedReader(fReader);
	      	String line = null;
	      	while (null != (line = bReader.readLine())) {
	      		if (null != aCache) {
	      			aCache.write(line.getBytes());
	      			aCache.write(LINE_FEED);
	      		}
		      	int semiColonIdx = line.lastIndexOf(":");
		      	int dashIdx = line.lastIndexOf("-");
		      	
		      	// see the format of PeerGuardianDatabaseEntry to understand the index check
		      	if (line.length() > 0 && semiColonIdx > 0 && dashIdx > 0 && (dashIdx > semiColonIdx)) {
		      		result.add(toIpRange(line));
		        }
	      	}
	      	bReader.close();
	      	fReader.close();
	      	toRead.delete();
	      	
	      	/*
		      System.out.println("memory before parsing entries: "
		  				+ ((double)Runtime.getRuntime().totalMemory() / 1000000D) + "M");
		      //result = parsePeerGuardianDatabaseEntries(peerGuardianEntries);
		      System.out.println("memory after parsing entries: "
		  				+ ((double)Runtime.getRuntime().totalMemory() / 1000000D) + "M");
		  		*/
		      /*
		      // safety margin, save if at least 100 entries are available. This avoids overwriting the cache
		      // file if the database (more likely to be an HTML page) contains junk data or is currently empty. 
		      if (result.size() >= 100) {
			      // entries were parsed successfuly, now cache can to saved to file
			      String filename = aConfigEntry.getConfigPath() + aConfigEntry.getProperty(SafepeerConfigConstants.CACHE_FILE_KEY);
			      File cacheFile = new File(filename);
			      saveDataToCacheFile(cache.toString(), cacheFile, aConfigEntry);
		      }
		      */
      	}
      } else {
      	//log.warn("No data could be retrieved to be parsed");
      }
    } catch (Exception ex) {
      ex.printStackTrace();
      //log.error("The database could not be parsed");
      //log.error(ex.toString());
      result = Collections.EMPTY_LIST;
    }
    return result;
  }
  
  /** Parse the given PeerGuardian database entries
   * @param someEntries  a list of the PeerGuardian database entries (PeerGuardianDatabaseEntry)
   * @return  a list of EvilPeerEntriey objects parsed, cannot be null
   */
  protected IpRange toIpRange(String anEntry) {
    IpRange retVal = null;
    try {
      // these indexes should be valid, checked in parse(...)
      int semiColonIdx = anEntry.lastIndexOf(":");
      int dashIdx = anEntry.lastIndexOf("-");
      String organization = anEntry.substring(0, semiColonIdx).trim();
      String lowerBound = anEntry.substring(semiColonIdx + 1, dashIdx).trim();
      String upperBound = anEntry.substring(dashIdx + 1).trim();
      
      // An IP address has at most 15 characters: xxx.xxx.xxx.xxx
      if (lowerBound.length() > 15 || upperBound.length() > 15) {
      	throw new IllegalArgumentException("A bound of the IP range is an invalid IP");
      }
      
      retVal = new IpRange(organization, lowerBound, upperBound);
    } catch (Exception ex) {
    	ex.printStackTrace();
      //log.warn("The following PeerGuardianDatabaseEntry could not be parsed: " + entry.toString());
      //log.warn(ex.toString());
    }
    return retVal;
  }
  
  /** Save the data to the cache file
   * @param someData  the data to be saved to the cache file
   * @param aFile the cache file
   * @param aConfigEntry  the config entry to get the config data from
   */
  /*
  protected void saveDataToCacheFile(String someData, File aFile, ConfigEntry aConfigEntry) {
    //MiniLogger log = DefaultMiniLogger.getInstance(SafePeer.LOG_IDENTIFIER);
    if (null != someData) {
      try {
      	String safeCache = aConfigEntry.getProperty(SafepeerConfigConstants.ENABLE_SAFECACHE);
      	boolean isSafeCacheEnabled = Boolean.valueOf(safeCache).booleanValue();
      	
      	if (isSafeCacheEnabled) {
      		int safeCacheSize = Integer.parseInt(aConfigEntry.getProperty(SafepeerConfigConstants.SAFECACHE_SIZE));
      		BufferedReader reader = new BufferedReader(new StringReader(someData));
      		int dataSize = 0;
      		while (null != reader.readLine()) {
      			dataSize++;
      		}
      		reader.close();
      		if (dataSize < safeCacheSize) {
      			//log.info("IP cache was not updated, there are less IPs than the defined minimum size: " + dataSize);
      			return;
      		}
      	}
      	
        FileOutputStream fos = new FileOutputStream(aFile);
        fos.write( someData.getBytes() );
        fos.flush();
        fos.close();
        //log.info("Database successfuly saved to IP cache");
      } catch (Exception ex) {
        //log.warn("The ip cache could not be saved");
        //log.warn(ex.toString());
      }
    } else {
      //log.warn("Cannot save data to cache file, content is null");
    }
  }
  */
  
}
